package diffcad;

import futils.Print;
import gui.Colorable;
import observers.ObservableDouble;

import java.awt.*;


public abstract class Shape extends Computation implements Colorable {
    static public final int shapeRadius = 5;

    private Color color = Color.black;
    private Color oldColor;
    point p1 = new point();
    point p2 = new point();
    point p3 = new point();
    point p4 = new point();
    point pc = new point();
    point pb = new point();

    Xform center = new Xform(120, -120, 1, -1);
    Xform display = center;

    // an overridden default, in degrees
    public ObservableDouble rotation =
            new ObservableDouble(45, "rotation");


    void pushGraphicsState(Graphics g) {
        oldColor = g.getColor();
        g.setColor(color);
    }

    void popGraphicsState(Graphics g) {
        g.setColor(oldColor);
    }

    abstract void draw(Graphics g);

    void point_toward(double theta) {
        double h = 10000;

        point p = new point();
        double thetad = theta / Pi_on_180;
        // p is an end-point of a line which starts at p1 and
        // ends at p. The angle is ccw from normal to grating.
        // System.out.println("New point toward called with"+thetad);
        p2.x = h * Math.cos(PI / 2 - theta) + p1.x;
        p2.y = h * Math.sin(PI / 2 - theta) + p1.y;
    }

    public void setColor(Color c) {
        color = c;
    }

    public Color getColor() {
        return color;
    }

    point dilate(double t) {
        Ray r = new Ray();
        r.p1 = p1;
        r.p2 = p2;
        return r.dilation(t);
    }

    point mouse_scale(point mouse) {
        point pm = new point();
        pm.x = (mouse.x - display.translate.x) / display.scale.x;
        pm.y = (mouse.y - display.translate.y) / display.scale.x;
        return pm;
    }

    int round(double d) {
        return ((int) Math.round(d));
    }

    int sx(double x) {
        return round(display.scale.x * x + display.translate.x);
    }

    int sy(double y) {
        return round(display.scale.y * y - display.translate.y);
    }


    void print() {
        Print.className(this);
        System.out.println("Rotation = " + rotation + " degrees");
        print("pc =");
        pc.print();
        print("pb =");
        pb.print();
        print("p1 =");
        p1.print();
        print("p2 =");
        p2.print();
        print("p3 =");
        p3.print();
        print("p4 =");
        p4.print();
    }


    void line(Graphics g, point p1, point p2) {
        g.drawLine(
                sx(p1.x),
                sy(p1.y),
                sx(p2.x),
                sy(p2.y));
    }


    void string(Graphics g, String str, point p) {
        g.drawString(str, sx(p.x), sy(p.y));
    }

    void fillCircle(Graphics g, point p, int radius) {
        g.fillOval(sx(p.x),
                sy(p.y), radius, radius);
    }

    void pointRotationdep(point p) {
        double tx,ty, c_theta, s_theta, theta;
        theta = rotation.getValue() * Pi_on_180;
        // rotate point p about pc an amount of r degrees
        // return the modified point
        c_theta = Math.cos(theta);
        s_theta = Math.sin(theta);
        tx = pc.x + (p.x - pc.x) * c_theta - (p.y - pc.y) * s_theta;
        ty = pc.y + (p.y - pc.y) * c_theta + (p.x - pc.x) * s_theta;
        p.x = tx;
        p.y = ty;
    }

    public void pointRotation(point p) {
        pointRotation(p, pc, rotation.getValue() * Pi_on_180);
    }

    public void pointRotation(point p, point pc, double theta) {
        // rotate point p about pc an amount of theta radians
        // return the modified point
        double c_theta = Math.cos(theta);
        double s_theta = Math.sin(theta);
        double tx = pc.x + (p.x - pc.x) * c_theta - (p.y - pc.y) * s_theta;
        double ty = pc.y + (p.y - pc.y) * c_theta + (p.x - pc.x) * s_theta;
        p.x = tx;
        p.y = ty;
    }


    double r_angle_r() {
        Ray r = new Ray();
        r.p1 = p1;
        r.p2 = p2;
        return r.r_angle_r();
    }

    double r_angle_d() {
        return radians_to_degrees(r_angle_r());
    }


    double i_angle_d(double p) {
        return radians_to_degrees(i_angle_r(p));
    }

    double i_angle_r(double p) {
        double i;
        double arg_to_asin;
        //System.out.println("n= "+n);
        double sin_r = Math.sin(r_angle_r());
// System.out.println("Pitch is = "+p);
        arg_to_asin = Geometry.n.getValue() * Geometry.lambda.getValue() / p - sin_r;

        if (r_angle_r() > 0) {
            //	System.out.println(" r>0, n = "+n+" Am inverting order");
            arg_to_asin = -Geometry.n.getValue() * Geometry.lambda.getValue() / p - sin_r;

        }

        if (Math.abs(arg_to_asin) >= 1) {
            System.out.println(
                    "I could not condition the arg to asine");
        }

        i = Math.asin(arg_to_asin);
        return i;
    }

    static void print(double d) {
        Print.print(d);
    }

    static void print(String str) {
        Print.print(str);
    }
    // inherits the implementation from the print class.


} // End of shape class




