package dumper;


import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;


/**

 * This class defines a FieldInfo in the class file. Fields are used to

 * describe instance variables in a class. The toString() method is

 * augmented by a version that takes an array of ConstantPoolInfo

 * objects (a constant pool). When a constant pool is available the

 * toString() method generates a declaration for the field as it would

 * appear in Java source.

 *

 * @version 	1.3, 16 Aug 1995

 * @author	Chuck McManis

 * @see		ClassFile

 */


public class FieldInfo {

    private short accessFlags;

    private ConstantPoolInfo name;

    private ConstantPoolInfo signature;

    AttributeInfo attributes[];


    public boolean read(DataInputStream di, ConstantPoolInfo pool[])

            throws IOException {

        int count;


        setAccessFlags(di.readShort());

        setName(pool[di.readShort()]);

        setSignature(pool[di.readShort()]);

        count = di.readShort();

        if (count != 0) {

            attributes = new AttributeInfo[count];

            for (int i = 0; i < count; i++) {

                attributes[i] = new AttributeInfo();

                if (!attributes[i].read(di, pool))

                    return (false);

            }

        }

        return (true);

    }


    public void write(DataOutputStream dos, ConstantPoolInfo pool[])

            throws IOException, Exception {

        dos.writeShort(getAccessFlags());

        dos.writeShort(ConstantPoolInfo.indexOf(getName(), pool));

        dos.writeShort(ConstantPoolInfo.indexOf(getSignature(), pool));

        if (attributes == null) {

            dos.writeShort(0);

        } else {

            dos.writeShort(attributes.length);

            for (int i = 0; i < attributes.length; i++) {

                attributes[i].write(dos, pool);

            }

        }

    }


    public String toString() {

        StringBuffer x = new StringBuffer();


        x.append(ClassFile.accessString(getAccessFlags()));

        x.append(ClassFile.typeString(getSignature().toString(), getName().toString()));

        if (attributes != null) {

            x.append(" = " + attributes[0].toString());

        }

        return (x.toString());

    }


    public String toString(ConstantPoolInfo pool[]) {

        StringBuffer x = new StringBuffer();

        String mytype;


        x.append(ClassFile.accessString(getAccessFlags()));

        mytype = ClassFile.typeString(getSignature().toString(), getName().toString());

        x.append(mytype);

        if (attributes != null) {

            if (mytype.startsWith("boolean")) {

                x.append(" " + attributes[0].toBoolean(pool));

            } else

                x.append(" " + attributes[0].toString(pool));

        }

        return (x.toString());

    }

    public short getAccessFlags() {
        return accessFlags;
    }

    public void setAccessFlags(short accessFlags) {
        this.accessFlags = accessFlags;
    }

    public ConstantPoolInfo getName() {
        return name;
    }

    public void setName(ConstantPoolInfo name) {
        this.name = name;
    }

    public ConstantPoolInfo getSignature() {
        return signature;
    }

    public void setSignature(ConstantPoolInfo signature) {
        this.signature = signature;
    }

}

