package grapher;

import java.awt.*;

/*************************************************************************

 *************************************************************************/


class G2Dint extends Graph2D {

/*
**    Set to true when a rectangle is being dragged out by the mouse
*/
    protected boolean drag = false;
/*
**    User limits. The user has set the limits using the mouse drag option
*/
    protected boolean userlimits = false;

/*
**    Ths popup window for the cursor position command
*/
    private Gin cpgin = null;
/*
**    Ths popup window for the data point command
*/
    private Gin dpgin = null;
/*
**    The popup window to manually set the range
*/
    private Range range = null;
/*
**    Button Down position
*/
    private int x0,y0;
/*
**    Button Drag position
*/
    private int x1,y1;
/*
**    Previous Button Drag position
*/
    private int x1old, y1old;

/*
**    Attached Axis which must be registered with this class
**    These are the axis used to find the drag range.
**    If no axis are registered no mouse drag.
*/
    protected Axis xaxis;
    protected Axis yaxis;


/*
**    Create xaxis to be used for the drag scaling
*/
    public Axis createXAxis() {
        xaxis = super.createAxis(Axis.BOTTOM);
        return xaxis;
    }
/*
**    Create yaxis to be used for the drag scaling
*/
    public Axis createYAxis() {
        yaxis = super.createAxis(Axis.LEFT);
        return yaxis;
    }
/*
**    Attach axis to be used for the drag scaling
*/
    public void attachAxis(Axis a) {
        if (a == null) return;

        super.attachAxis(a);

        if (a.getAxisPos() == Axis.BOTTOM || a.getAxisPos() == Axis.TOP) {
            xaxis = a;
        } else {
            yaxis = a;
        }
    }
/*
**  Override the normal Graph2D update method with this new one
*/
    public void update(Graphics g) {
        Rectangle r = bounds();
        Color c = g.getColor();

        if (drag) {
/*
**         Drag out the new box.
**         Use drawLine instead of drawRect to avoid problems
**         when width and heights become negative. Seems drawRect
**         can't handle it!
*/

            g.setColor(getBackground());
            g.drawLine(x0, y0, x1old, y0);
            g.drawLine(x1old, y0, x1old, y1old);
            g.drawLine(x1old, y1old, x0, y1old);
            g.drawLine(x0, y1old, x0, y0);
            g.setColor(getForeground());
            g.drawLine(x0, y0, x1, y0);
            g.drawLine(x1, y0, x1, y1);
            g.drawLine(x1, y1, x0, y1);
            g.drawLine(x0, y1, x0, y0);


            g.setColor(c);

            x1old = x1;
            y1old = y1;

            return;
        }

        if (clearAll) {
            g.setColor(getBackground());
            g.fillRect(r.x, r.y, r.width, r.height);
            g.setColor(c);
        }
        if (paintAll) paint(g);
    }

/*
** Handle the events. Use the predefined methods for the Component class
*/
    public boolean keyDown(Event e, int key) {

        if (xaxis == null || yaxis == null) return false;

        switch (key) {

            case 'R':
                xaxis.resetRange();
                yaxis.resetRange();

                userlimits = false;

                repaint();
                return true;
            case 'r':
                repaint();
                return true;
            case 'c':
                if (cpgin == null) cpgin = new Gin("Position");
                if (cpgin.isVisible()) {
                    cpgin.hide();
                } else {
                    cpgin.show();
                }
                return true;
            case 'D':
                if (dpgin != null) dpgin.hide();
                return true;
            case 'd':
                if (dpgin == null) dpgin = new Gin("Data Point");
                dpgin.show();
                double d[] = getClosestPoint(e.x, e.y);
                dpgin.setXlabel(d[0]);
                dpgin.setYlabel(d[1]);
                int ix = xaxis.getInteger(d[0]);
                int iy = yaxis.getInteger(d[1]);
                if (ix >= datarect.x &&
                        ix <= datarect.x + datarect.width &&
                        iy >= datarect.y &&
                        iy <= datarect.y + datarect.height) {
                    Graphics g = getGraphics();
                    g.fillOval(ix - 4, iy - 4, 8, 8);
                }
                return true;
            case 'm':
                if (range == null) range = new Range(this);

                range.show();
                range.requestFocus();
                userlimits = true;
                return true;
            default:
//                             System.out.println("KeyPress "+e.key);
                return false;
        }

    }

    public boolean mouseDown(Event e, int x, int y) {

        if (xaxis == null || yaxis == null) return false;

        x0 = x;
        y0 = y;

        drag = true;
        x1old = x0;
        y1old = y0;


        if (x0 < datarect.x)
            x0 = datarect.x;
        else if (x0 > datarect.x + datarect.width)
            x0 = datarect.x + datarect.width;

        if (y0 < datarect.y)
            y0 = datarect.y;
        else if (y0 > datarect.y + datarect.height)
            y0 = datarect.y + datarect.height;


        return true;
    }

    public boolean mouseUp(Event e, int x, int y) {

        if (xaxis == null || yaxis == null) return false;

        x1 = x;
        y1 = y;

        if (drag) userlimits = true;

        drag = false;


        if (x1 < datarect.x)
            x1 = datarect.x;
        else if (x1 > datarect.x + datarect.width)
            x1 = datarect.x + datarect.width;

        if (y1 < datarect.y)
            y1 = datarect.y;
        else if (y1 > datarect.y + datarect.height)
            y1 = datarect.y + datarect.height;


        if (Math.abs(x0 - x1) > 5 && Math.abs(y0 - y1) > 5) {
            if (x0 < x1) {
                xaxis.minimum = xaxis.getDouble(x0);
                xaxis.maximum = xaxis.getDouble(x1);
            } else {
                xaxis.maximum = xaxis.getDouble(x0);
                xaxis.minimum = xaxis.getDouble(x1);
            }

            if (y0 > y1) {
                yaxis.minimum = yaxis.getDouble(y0);
                yaxis.maximum = yaxis.getDouble(y1);
            } else {
                yaxis.maximum = yaxis.getDouble(y0);
                yaxis.minimum = yaxis.getDouble(y1);
            }

            repaint();
        }
        return true;
    }

    public boolean mouseDrag(Event e, int x, int y) {

        if (xaxis == null || yaxis == null) return false;

        x1 = x;
        y1 = y;

        if (drag) {

            if (x1 < datarect.x)
                x1 = datarect.x;
            else if (x1 > datarect.x + datarect.width)
                x1 = datarect.x + datarect.width;

            if (y1 < datarect.y)
                y1 = datarect.y;
            else if (y1 > datarect.y + datarect.height)
                y1 = datarect.y + datarect.height;

        }

        if (cpgin != null && cpgin.isVisible()) {
            cpgin.setXlabel(xaxis.getDouble(x1));
            cpgin.setYlabel(yaxis.getDouble(y1));
        }

        repaint();

        return true;


    }

    public boolean mouseMove(Event e, int x, int y) {

        if (xaxis == null || yaxis == null) return false;

        x1 = e.x;
        y1 = e.y;


        if (cpgin != null && cpgin.isVisible()) {
            cpgin.setXlabel(xaxis.getDouble(x1));
            cpgin.setYlabel(yaxis.getDouble(y1));
        }

        return true;

    }

    public boolean action(Event e, Object a) {


        if (xaxis == null || yaxis == null) return false;

        if (e.target instanceof Range && range != null) {
            Double d;
            double txmin = xaxis.minimum;
            double txmax = xaxis.maximum;
            double tymin = yaxis.minimum;
            double tymax = yaxis.maximum;


            d = range.getXmin();
            if (d != null) txmin = d.doubleValue();
            d = range.getXmax();
            if (d != null) txmax = d.doubleValue();
            d = range.getYmin();
            if (d != null) tymin = d.doubleValue();
            d = range.getYmax();
            if (d != null) tymax = d.doubleValue();


            if (txmax > txmin && tymax > tymin) {
                xaxis.minimum = txmin;
                xaxis.maximum = txmax;
                yaxis.minimum = tymin;
                yaxis.maximum = tymax;
            }


            repaint();


            return true;
        }
        return false;
    }


/*
**   Find the closest data point to the cursor
*/
    protected double[] getClosestPoint(int ix, int iy) {
        DataSet ds;
        int i;
        double a[] = new double[3];
        double distsq = -1.0;
        double data[] = {0.0, 0.0};
        double x = xaxis.getDouble(ix);
        double y = yaxis.getDouble(iy);

        //System.out.println("getClosestPoint: x="+x+", y="+y);

        for (i = 0; i < dataset.size(); i++) {
            ds = (DataSet) (dataset.elementAt(i));

            a = ds.getClosestPoint(x, y);

            if (distsq < 0.0 || distsq > a[2]) {
                data[0] = a[0];
                data[1] = a[1];
                distsq = a[2];
            }
        }
        return data;

    }

}

/*
** Gin
**      Popup a window to output data after a Graphics Input command
*/

class Gin extends Frame {

    private Label xlabel = new Label();
    private Label ylabel = new Label();


    public Gin() {

        setLayout(new GridLayout(2, 1));

        xlabel.setAlignment(Label.LEFT);
        ylabel.setAlignment(Label.LEFT);

        this.setFont(new Font("Helvetica", Font.PLAIN, 20));

        add("x", xlabel);
        add("y", ylabel);

        resize(150, 100);
        super.setTitle("Graphics Input");
    }

    public Gin(String title) {
        this();
        if (title != null) super.setTitle(title);
    }

    public void setXlabel(double d) {
        xlabel.setText(String.valueOf(d));
    }

    public void setYlabel(double d) {
        ylabel.setText(String.valueOf(d));
    }

    public void setLabels(double dx, double dy) {
        xlabel.setText(String.valueOf(dx));
        ylabel.setText(String.valueOf(dy));
    }

    public void setFont(Font f) {

        if (f == null) return;
        xlabel.setFont(f);
        ylabel.setFont(f);

    }

    public void resize(int x, int y) {
        super.resize(x, y);
    }

    public boolean keyDown(Event e, int key) {

        if (key == 'h') {
            hide();
            return true;
        }

        return false;

    }

}

/*
** Range
**      Popup a window for the range of the plot
*/

class Range extends Frame {

    Graph2D g2d = null;


    private Label xminLabel = new Label("Xmin");
    private Label yminLabel = new Label("Ymin");
    private Label xmaxLabel = new Label("Xmax");
    private Label ymaxLabel = new Label("Ymax");

    private TextField xminText = new TextField(20);
    private TextField yminText = new TextField(20);
    private TextField xmaxText = new TextField(20);
    private TextField ymaxText = new TextField(20);

    private Button cancel = new Button("Cancel");
    private Button done = new Button("Done");


    public Range(Graph2D g) {

        g2d = g;

        setLayout(new GridLayout(5, 2, 5, 10));

        xminLabel.setAlignment(Label.LEFT);
        xmaxLabel.setAlignment(Label.LEFT);
        yminLabel.setAlignment(Label.LEFT);
        ymaxLabel.setAlignment(Label.LEFT);


        add("xminLabel", xminLabel);
        add("xminText", xminText);

        add("xmaxLabel", xmaxLabel);
        add("xmaxText", xmaxText);

        add("yminLabel", yminLabel);
        add("yminText", yminText);

        add("ymaxLabel", ymaxLabel);
        add("ymaxText", ymaxText);

        add("cancel", cancel);
        cancel.setBackground(Color.red);

        add("done", done);
        done.setBackground(Color.green);

        resize(250, 250);
        super.setTitle("Set Plot Range");
    }

    public Double getXmin() {
        try {
            return Double.valueOf(xminText.getText());
        } catch (Exception ex) {
            return null;
        }
    }

    public Double getXmax() {
        try {
            return Double.valueOf(xmaxText.getText());
        } catch (Exception ex) {
            return null;
        }
    }

    public Double getYmin() {
        try {
            return Double.valueOf(yminText.getText());
        } catch (Exception ex) {
            return null;
        }
    }

    public Double getYmax() {
        try {
            return Double.valueOf(ymaxText.getText());
        } catch (Exception ex) {
            return null;
        }
    }


    public void resize(int x, int y) {
        super.resize(x, y);
    }

    public void requestFocus() {
        xminText.requestFocus();
    }
/*
** Handle the events
*/
    public boolean keyDown(Event e, int key) {

        if (e.target instanceof TextField) {

            if ((key == 10 || e.key == 13)) {

                if (xminText.equals(e.target)) {
                    xmaxText.requestFocus();
                    return true;
                } else if (xmaxText.equals(e.target)) {
                    yminText.requestFocus();
                    return true;
                } else if (yminText.equals(e.target)) {
                    ymaxText.requestFocus();
                    return true;
                } else if (ymaxText.equals(e.target)) {
                    xminText.requestFocus();
                    return true;
                }


                return true;
            }

        }

        return false;

    }


    public boolean action(Event e, Object a) {

        if (e.target instanceof Button) {
            if (done.equals(e.target) && g2d != null) {
                g2d.deliverEvent(new Event(this, Event.ACTION_EVENT, this));
                hide();
                return true;
            } else if (cancel.equals(e.target)) {
                hide();
                return true;
            }
        }


        return false;
    }


}
