package ipl;

import java.awt.*;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;

public class PixelPlane {

    private int width;
    private int height;
    ColorModel cm;
    public int pels[];


    PixelPlane(int w, int h) {
        width = w;
        height = h;
        pels = new int[w * h];
        cm = ColorModel.getRGBdefault();
    }

    PixelPlane(double w, double h) {
        width = (int) w;
        height = (int) h;
        pels = new int[width * height];
        cm = ColorModel.getRGBdefault();
    }

    public Image makeImage() {
        return
                Toolkit.getDefaultToolkit().createImage(
                        new MemoryImageSource(
                                getWidth(),
                                getHeight(),
                                ColorModel.getRGBdefault(),
                                pels, 0,
                                getWidth()));
    }

    public int getLength() {
        return pels.length;
    }

    public int getHeight() {
        return height;
    }

    public int getWidth() {
        return width;
    }

    public int getRed(int i) {
        return cm.getRed(pels[i]);
    }

    public int getGreen(int i) {
        return cm.getGreen(pels[i]);
    }

    public int getBlue(int i) {
        return cm.getBlue(pels[i]);
    }

    public int getAlpha(int i) {
        return cm.getAlpha(pels[i]);
    }


    public int getRed(int x, int y) {
        if (inrange(x, y)) {
            return cm.getRed(pels[y * width + x]);
        } else {
            return 0;
        }
    }

    public int getGreen(int x, int y) {
        if (inrange(x, y)) {
            return cm.getGreen(pels[y * width + x]);
        } else {
            return 0;
        }

    }

    public int getBlue(int x, int y) {
        if (inrange(x, y)) {
            return cm.getBlue(pels[y * width + x]);
        } else {
            return 0;
        }

    }

    public int getAlpha(int x, int y) {
        if (inrange(x, y)) {
            return cm.getAlpha(pels[y * width + x]);
        } else {
            return 0;
        }
    }

    public PixelPlane copy() {
        PixelPlane newPlane =
                new PixelPlane(
                        getWidth(),
                        getHeight());
        System.arraycopy(
                pels, 0,
                newPlane.pels, 0, pels.length);
        return newPlane;
    }

    boolean inrange(int x, int y) {
        return ((x >= 0) && (y >= 0) && (x < width) && (y < height));
    }

    public int maximumRadius() {
        return width * width * height * height;
    }

    private static void waitForImage(Component component,
                                     Image image) {
        MediaTracker tracker = new MediaTracker(component);
        try {
            tracker.addImage(image, 0);
            tracker.waitForAll();
            tracker.waitForID(0);
            if (tracker.checkID(0))
                System.out.println("The image checked as loaded");
            else
                System.out.println("Load failure!");
        } catch (InterruptedException e) {
            Assert.notNull(null);
        }
    }


    public int MakePixel(int r, int g, int b, int a) {
        //if (r > 255) r = 255;
        //if (g > 255) g = 255;
        //if (b > 255) b = 255;
        return (a << 24) | (r << 16) | (g << 8) | b;
    }

    public void setPixel(int x, int y, int r, int g, int b, int a) {
        try {
            pels[y * width + x] = MakePixel(r, g, b, a);
        } catch (ArrayIndexOutOfBoundsException e) {
            // tuff buggers
        }
    }

    public void setPixel(int i, int r, int g, int b, int a) {
        try {
            pels[i] = MakePixel(r, g, b, a);
        } catch (ArrayIndexOutOfBoundsException e) {
            // tuff buggers
        }
    }

    // sets a packed pel
    public void setPixel(int x, int y, int pel) {
        try {
            pels[y * width + x] = pel;
        } catch (ArrayIndexOutOfBoundsException e) {
            // tuff buggers
        }
    }

// gets a packed pel
    public int getPixel(int x, int y) {
        return pels[y * width + x];
    }

    public int getPixel(int i) {
        return pels[i];
    }


    public void printSize() {
        System.out.println(
                "W:" +
                width +
                " H:" +
                height);
    }

}