package observers;

import java.awt.*;
import java.util.Observable;
import java.util.Observer;

//
// DoubleScrollbar - a Scrollbar that modifies an
// ObservableDouble.  This class functions as both a
// "view" of the observable, since the position of
// the scrollbar is changed as the observable's value
// is changed, and it is a "controller" since it also
// sets the value of the observable.
//
// DoubleScrollbar has the same constructors as Scrollbar,
// except that in each case, there is an additional
// parameter that is the ObservableDouble.
// Note:  On the constructor where you pass in the initial
// scrollbar position, the position is ignored.

public class DoubleScrollbar extends Scrollbar implements Observer {
    private ObservableDouble doubleValue;
// The bulk of this class is implementing the various
// constructors that are available in the Scrollbar class.
    public DoubleScrollbar(ObservableDouble newValue) {
        super();     // Call the Scrollbar constructor
        doubleValue = newValue;
        doubleValue.addObserver(this);        // Register interest
        setValue((int) doubleValue.getValue());     // Change scrollbar position
    }

    public DoubleScrollbar(ObservableDouble newValue, int orientation) {
        super(orientation);         // Call the Scrollbar constructor
        doubleValue = newValue;
        doubleValue.addObserver(this);        // Register interest
        setValue((int) doubleValue.getValue());     // Change scrollbar position
    }

    public DoubleScrollbar(ObservableDouble newValue, int orientation,
                           int value, int pageSize, int lowValue, int highValue) {
        super(orientation, value, pageSize, lowValue, highValue);
        doubleValue = newValue;
        doubleValue.addObserver(this);        // Register interest
        setValue((int) doubleValue.getValue());     // Change scrollbar position
    }
// The handleEvent method check with the parent class (Scrollbar) to see
// if it wants the event, if not, just assume the scrollbar value has
// changed and update the observable int with the new position.
    public boolean handleEvent(Event evt) {
        if (super.handleEvent(evt)) {
            return true;     // The Scrollbar class handled it

        }
        doubleValue.setValue(getValue());     // Update the observable int
        return true;
    }
// update is called whenever the observable int changes its value
    public void update(Observable obs, Object arg) {
        setValue((int) doubleValue.getValue());
    }
}