package surfer;

import java.applet.Applet;
import java.awt.*;


public class Surface extends Applet implements Runnable {

    private FunctionPanel function_panel;          // the function panel

    private SettingPanel setting_panel;            // the setting panel

    private SurfaceCanvas canvas;                  // the drawing canvas

    private Parser parser1,parser2;                // the function parser

    private boolean func1calc, func2calc;          // calculated flag

    private Thread thread;                         // current thread

    private int calc_divisions;                    // number of divisions to calculate


    public static final String APP_NAME = "Surfer";

    public static final String APP_VERSION = "Version 1.20";


    private static final int INIT_CALC_DIV = 20;

    private static final int INIT_DISP_DIV = 20;



    /*--------------------------------------------------------------------------------

                              Returns the applet information

      --------------------------------------------------------------------------------*/



    public String getAppletInfo() {

        return APP_NAME + " " + APP_VERSION + "\n" +

                "Copyright (c) 1996 Yanto Suryono\n" +

                "Compiled: October 14 1996\n\n";

    }



    /*--------------------------------------------------------------------------------

                                  Initializes the applet

      --------------------------------------------------------------------------------*/



    public void init() {

        thread = new Thread(this);

        parser1 = new Parser(2);

        parser1.defineVariable(1, "x");

        parser1.defineVariable(2, "y");

        parser2 = new Parser(2);

        parser2.defineVariable(1, "x");

        parser2.defineVariable(2, "y");

        setFont(new Font("Helvetica", Font.PLAIN, 11));

        setBackground(Color.lightGray);

        setLayout(new BorderLayout());

        add("North", function_panel = new FunctionPanel());

        add("East", setting_panel = new SettingPanel(INIT_CALC_DIV, INIT_DISP_DIV));

        add("Center", canvas = new SurfaceCanvas(this));

        setting_panel.setRanges(-3, -3, -1, 3, 3, 1);

    }



    /*--------------------------------------------------------------------------------

                               Sets the status information

      --------------------------------------------------------------------------------*/



    public void setMessage(String message) {

        function_panel.setMessage(message);

    }



    /*--------------------------------------------------------------------------------

                        Calculates function values at sampling points

      --------------------------------------------------------------------------------*/



    public void run() {

        float stepx, stepy, x, y, v;

        float xi,xx,yi,yx;

        float min, max;

        boolean f1, f2;

        int i,j,k,total;


        try {

            xi = Float.valueOf(setting_panel.getXMin()).floatValue();

            yi = Float.valueOf(setting_panel.getYMin()).floatValue();

            xx = Float.valueOf(setting_panel.getXMax()).floatValue();

            yx = Float.valueOf(setting_panel.getYMax()).floatValue();

            if ((xi >= xx) || (yi >= yx)) throw new NumberFormatException();

        } catch (NumberFormatException e) {

            setMessage("Error in ranges");

            return;

        }


        canvas.setRanges(xi, xx, yi, yx);

        setMessage("parsing ...");


        f1 = function_panel.isFunction1Selected();

        if (f1) {

            parser1.define(function_panel.getFunction1Definition());

            parser1.parse();


            if (parser1.getErrorCode() != parser1.NO_ERROR) {

                setMessage("Parse error: " + parser1.getErrorString() +

                        " at function 1, position " + parser1.getErrorPosition());

                return;

            }

        }


        f2 = function_panel.isFunction2Selected();

        if (f2) {

            parser2.define(function_panel.getFunction2Definition());

            parser2.parse();


            if (parser2.getErrorCode() != parser2.NO_ERROR) {

                setMessage("Parse error: " + parser2.getErrorString() +

                        " at function 2, position " + parser2.getErrorPosition());

                return;

            }

        }


        if (!f1 && !f2) {

            setMessage("No function selected");

            return;

        }


        thread.yield();

        calc_divisions = setting_panel.getCalcDivisions();

        canvas.setDataAvailability(false);

        func1calc = f1;
        func2calc = f2;


        stepx = (xx - xi) / calc_divisions;

        stepy = (yx - yi) / calc_divisions;

        total = (calc_divisions + 1) * (calc_divisions + 1);



        // release memory being used



        canvas.setValuesArray(null);

        SurfaceVertex[][] vertex = null;



        /* The following program:



           SurfaceVertex[][] vertex = new SurfaceVertex[2][];



           if (f1) vertex[0] = new SurfaceVertex[total];

           if (f2) vertex[1] = new SurfaceVertex[total];





           Didn't work with my Microsoft Internet Explorer v3.0b2.

           It resulted in a "java.lang.ArrayStoreException"  :(



         */



        try {

            vertex = new SurfaceVertex[2][total];

            if (!f1) vertex[0] = null;

            if (!f2) vertex[1] = null;

        } catch (OutOfMemoryError e) {

            setMessage("Not enough memory");

            return;

        } catch (Exception e) {

            setMessage("Error: " + e.toString());

            return;

        }


        max = Float.POSITIVE_INFINITY;

        min = Float.NEGATIVE_INFINITY;

        setting_panel.setMinimumResult("");

        setting_panel.setMaximumResult("");


        canvas.destroyImage();


        i = 0;
        j = 0;
        k = 0;
        x = xi;
        y = yi;


        float xfactor = 20 / (xx - xi);

        float yfactor = 20 / (yx - yi);


        while (i <= calc_divisions) {

            if (f1) {

                parser1.setVariable(1, x);

                parser1.setVariable(2, y);

            }

            if (f2) {

                parser2.setVariable(1, x);

                parser2.setVariable(2, y);

            }

            while (j <= calc_divisions) {

                thread.yield();

                if (f1) {

                    v = (float) parser1.evaluate();

                    if (Float.isInfinite(v)) v = Float.NaN;

                    if (!Float.isNaN(v)) {

                        if ((v > max) || (Float.isInfinite(max)))
                            max = v;
                        else if ((v < min) || (Float.isInfinite(min))) min = v;

                    }

                    vertex[0][k] = new SurfaceVertex((x - xi) * xfactor - 10,

                            (y - yi) * yfactor - 10, v);

                }

                if (f2) {

                    v = (float) parser2.evaluate();

                    if (Float.isInfinite(v)) v = Float.NaN;

                    if (!Float.isNaN(v)) {

                        if ((v > max) || (Float.isInfinite(max)))
                            max = v;
                        else if ((v < min) || (Float.isInfinite(min))) min = v;

                    }

                    vertex[1][k] = new SurfaceVertex((x - xi) * xfactor - 10,

                            (y - yi) * yfactor - 10, v);

                }

                j++;
                y += stepy;

                if (f1) parser1.setVariable(2, y);

                if (f2) parser2.setVariable(2, y);

                k++;

                setMessage("Calculating : " + k * 100 / total + "% completed");

            }

            j = 0;
            y = yi;
            i++;
            x += stepx;

        }


        if (Float.isInfinite(min)) min = Float.NaN;

        if (Float.isInfinite(max)) max = Float.NaN;

        setting_panel.setMinimumResult(Float.toString(min));

        setting_panel.setMaximumResult(Float.toString(max));

        canvas.setValuesArray(vertex);

        canvas.setDataAvailability(true);

        canvas.repaint();

    }



    /*--------------------------------------------------------------------------------

                                   Handles user actions

      --------------------------------------------------------------------------------*/



    public boolean action(Event e, Object arg) {

        if (e.target instanceof Button) {

            if (arg.equals("Calculate")) {

                if (!thread.isAlive()) {

                    thread = new Thread(this);

                    thread.start();

                }

            } else if (arg.equals("Stop")) {

                if (thread.isAlive()) {

                    thread.stop();

                    try {

                        thread.join();

                    } catch (InterruptedException exception) {
                    }
                    ;

                    setMessage("Interrupted");

                    canvas.setDataAvailability(false);

                    canvas.repaint();

                }

                canvas.interrupt();

            }

        } else if (e.target instanceof Choice) {  // new plot mode or settings

            // has been selected

            canvas.destroyImage();

            canvas.repaint();

        } else if (e.target instanceof Checkbox) {     // settings have changed

            canvas.destroyImage();

            canvas.repaint();

        } else

            return super.action(e, arg);

        return true;

    }



    /*--------------------------------------------------------------------------------

                                       Starts applet

      --------------------------------------------------------------------------------*/



    public void start() {

        repaint();

    }



    /*--------------------------------------------------------------------------------

                                 Stops applet's activities

      --------------------------------------------------------------------------------*/



    public void stop() {

        if (thread.isAlive()) {

            thread.stop();

            try {

                thread.join();

            } catch (InterruptedException exception) {
            }
            ;

            setMessage("Stopped");

            canvas.setDataAvailability(false);

        }

        canvas.interrupt();

    }



    /*--------------------------------------------------------------------------------

                Returns whether Surface Plotter should delay regeneration

                         until mouse buttons have been released

      --------------------------------------------------------------------------------*/



    public boolean isExpectDelay() {

        return function_panel.isExpectDelay();

    }



    /*--------------------------------------------------------------------------------

                       Functions that return/set plotting variables

          These functions are called ONLY from SurfaceCanvas's importVariables()

      --------------------------------------------------------------------------------*/



    public boolean isBoxed() {

        return setting_panel.isBoxed();

    }


    public boolean isMesh() {

        return setting_panel.isMesh();

    }


    public boolean isScaleBox() {

        return setting_panel.isScaleBox();

    }


    public boolean isDisplayXY() {

        return setting_panel.isDisplayXY();

    }


    public boolean isDisplayZ() {

        return setting_panel.isDisplayZ();

    }


    public boolean isDisplayGrids() {

        return setting_panel.isDisplayGrids();

    }


    public boolean isPlotFunction1() {

        return (func1calc && function_panel.isFunction1Selected());

    }


    public boolean isPlotFunction2() {

        return (func2calc && function_panel.isFunction2Selected());

    }


    public int getPlotMode() {

        return function_panel.getPlotMode();

    }


    public int getCalcDivisions() {

        return calc_divisions;

    }



    /*--------------------------------------------------------------------------------

                    Returns current Z ranges needed by SurfaceCanvas

      --------------------------------------------------------------------------------*/



    public String getZMin() {

        return setting_panel.getZMin();

    }


    public String getZMax() {

        return setting_panel.getZMax();

    }



    /*--------------------------------------------------------------------------------

                      Gets / sets number of divisions to display

      --------------------------------------------------------------------------------*/



    public int getDispDivisions() {

        int plot_density;


        plot_density = setting_panel.getDispDivisions();

        if (plot_density > calc_divisions) plot_density = calc_divisions;

        while ((calc_divisions % plot_density) != 0) plot_density++;

        return plot_density;

    }


    public void setDispDivisions(String value) {

        setting_panel.setDispDivisions(value);

    }



    /*--------------------------------------------------------------------------------

           Routines to give Surface Plotter a frame when running as an application

      --------------------------------------------------------------------------------*/



    public static void main(String args[]) {

        new SurfaceFrame(new Surface(), "Surface Plotter");

    }

}



