/**
 * Victor Silva - University of Bridgeport CS411X
 *
 * This user interface is used to test the GIF/JPG read/write
 * and image processing functions.
 *
 * Revision History:
 *    Ver 0.1 - Preliminary user interface with menus giving
 *                user access to the various functions.
 *    Ver 0.2 - GIF image writing added, it's very ssslooow.
 *                Two seconds for the infamous lyon.gif.
 *                Many improvements are possible and will be
 *                submitted for next status report.
 *              RGB to CMY added.
 *    Ver 0.3 - Added FileDialog to select GIF filename to write.
 *
 *    20 March 1997
 *    Ver 0.4 - Removed the need for having two Image classes.
 *              Now the image that is opened is first converted
 *              to ARGB values in an array of doubles.
 *              In ver. 0.3 the image was stored in an Image
 *              class after each operation, requiring a "PixelGrab"
 *              each time an operation on an image was performed.
 *              In this version a "PixelGrab" is done only once
 *              when a new image file is opened.  After the file
 *              is opened the image is put in a doubles array
 *              and all operations are performed on that array.
 *              Added RGB to CMY.
 *              Added HLS to RGB and RGB to HLS.
 *              Added HSV to RGB and RGB to HSV.
 *
 *    07 April 1997
 *    Ver 0.5 - Fixed bug that occurred if the same image was reloaded.
 *              When the same image is reloaded prepareImage will return
 *              immediately with a value of true indicating that the
 *              image was already prepared (from last load).
 *              Therefore if prepareImage returns "true", the new code
 *              grabs the image pixels and reloads the image
 *              arrays.
 *              Converting Ver 0.4 from R,G,B stored as double to
 *              R,G,B stored as int was a bear. The program runs
 *              much faster but I'm not satisfied with the results.
 *              Applying a color space conversion results in lossy
 *              conversion, i.e. converting RGB to CMY to RGB does
 *              not result in the original data back as the double data
 *              conversion did.
 *              Added PPM image saving. It's very fast, 20 times faster
 *              than saving GIFs.
 *
 *    27 April 1997
 *    Ver 0.6 - Added some translation routines to the menu.
 *              Rotation and zoom.  There is a problem with
 *              entering the required translation factor,
 *              for example angle of rotation, after it is
 *              entered into a DoubleDialog (from DiffCAD)
 *              the image must be clicked with the mouse pointer
 *              before the keyboard event will work.
 *              Currently supported keyboard events are:
 *                'r' - CW rotation
 *                'z' - Zoom (both in and out)
 *
 *              Added histogram frames for each primary color.
 *
 *
 */
package vs;

import gui.ClosableFrame;
import ipl.ProcessPlane;

import java.awt.*;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;


public class vsImage extends ClosableFrame {
    private ProcessPlane pp;
    private MenuBar mbar;
    private MenuItem openItem, saveAsGIF, saveAsPPM;
    private MenuItem convRGB2HSV, convHSV2RGB, convRGB2HLS,
    convHLS2RGB, convRGB2CMY, convCMY2RGB,
    doTranslate, doHistogram, doFFT, doIFFT,
    doLogPSD, doNoLogPSD,
    doExecuteFilter, doEnhanceEdges;


    private MenuItem quitItem;
    private MenuItem aboutItem, doTestFFT;

    // For rotation and zooming
    ObservableDouble ang = new ObservableDouble();

    ColorUtils CU = new ColorUtils();
    ImageUtils iUtil = new ImageUtils();
    FFT imgFFT = new FFT();

    // We want only one of these to exist
    ImageScroller imgScroll = null;
    Image img;
    int imageWidth;
    int imageHeight;
    int numPixels;

    // Standard Java ARGB pixel.
    int[] imageData;
    // Each color pixel is a short.
    short[] imageData_R;
    short[] imageData_G;
    short[] imageData_B;


    float[] redHLS_HSV;
    float[] greenHLS_HSV;
    float[] blueHLS_HSV;

    public static void main(String args[]) {
        vsImage iMenu = new vsImage();
        iMenu.reshape(500, 500, 250, 250);
        iMenu.show();

    }


    public static void waitForImage(Component component, Image image) {
        MediaTracker tracker = new MediaTracker(component);
        try {
            tracker.addImage(image, 0);
            tracker.waitForID(0);
        } catch (InterruptedException e) {
            Assert.notNull(null);
        }
    }

    static public String openFile() {
        Frame f = new Frame();
        FileDialog dialog = new FileDialog(f, "Select file");
        dialog.setFile("*.gif;*.jpg;*.jpeg;*.ppm");

        dialog.show();
        String file_name = dialog.getFile();
        String path_name = dialog.getDirectory();
        String file_string = path_name + file_name;
        System.out.println("Opening file: " + file_string);
        dialog.dispose();

        return (file_string);
    }

    public Image getImage() {
        // Open up the image, by file name and wait for it!
        String fileName = openFile();
        Image Img;

        if (fileName.endsWith(".ppm")) {
            System.out.println("PPM image selected");
            try {
                imageData = ReadPPM.DoIt(fileName);
                imageWidth = ReadPPM.width;
                imageHeight = ReadPPM.height;
            } catch (Exception e) {
                System.out.println("Read PPM Exception!");
            }
            imageData_R = CU.getRedArray(imageData);
            imageData_G = CU.getGreenArray(imageData);
            imageData_B = CU.getBlueArray(imageData);
            img = getToolkit().createImage(
                    new MemoryImageSource(imageWidth, imageHeight,
                            ColorModel.getRGBdefault(), imageData, 0, imageWidth));
        } else {
            img = Toolkit.getDefaultToolkit().getImage(fileName);
            waitForImage(this, img);
        }

        // move the image, change its dimensions
        reshape(100, 100, img.getWidth(this), img.getHeight(this));
        show();

        return img;
    }

    public vsImage(Image externalImage) {
        super("Image Manipulator");

        init();
        img = externalImage;
        grabImageDisplayFrame();

        // move the image, change its dimensions
        reshape(100, 100, img.getWidth(this), img.getHeight(this));
        show();
    }

    // make a vsimage with no frame, for encpasulation
    // via containment.
    public vsImage(Image externalImage, String title) {
        super(title);
        img = externalImage;
        grabPixels();
    }

    public vsImage() {
        super("Image Manipulator");

        init();
    }


    // set up the menu bars and alter the frame!
    public void init() {

        MenuBar mbar = getMenuBar();
        if (mbar == null)
            mbar = new MenuBar();
        Menu fileMenu = new Menu("File", true);

        fileMenu.add(openItem = new MenuItem("Open"));
        fileMenu.add(saveAsPPM = new MenuItem("Save as PPM"));
        fileMenu.add(saveAsGIF = new MenuItem("Save as GIF"));
        fileMenu.addSeparator();
        fileMenu.add(quitItem = new MenuItem("Quit"));
        mbar.add(fileMenu);
        setMenuBar(mbar);

        Menu procMenu = new Menu("Function", true);
        procMenu.add(convRGB2HSV = new MenuItem("RGBtoHSV"));
        procMenu.add(convHSV2RGB = new MenuItem("HSVtoRGB"));
        procMenu.add(convRGB2HLS = new MenuItem("RGBtoHLS"));
        procMenu.add(convHLS2RGB = new MenuItem("HLStoRGB"));
        procMenu.add(convRGB2CMY = new MenuItem("RGBtoCMY"));
        procMenu.add(convCMY2RGB = new MenuItem("CMYtoRGB"));
        procMenu.addSeparator();
        procMenu.add(doTranslate = new MenuItem("Translation"));
        procMenu.add(doHistogram = new MenuItem("Histogram"));
        procMenu.addSeparator();
        procMenu.add(doFFT = new MenuItem("FFT"));
        procMenu.add(doIFFT = new MenuItem("IFFT"));
        procMenu.add(doLogPSD = new MenuItem("Use Log PSD"));
        procMenu.add(doNoLogPSD = new MenuItem("Use Only PSD"));
        procMenu.addSeparator();
        procMenu.add(doExecuteFilter = new MenuItem("Execute Filter"));
        procMenu.add(doEnhanceEdges = new MenuItem("Enhance Edges"));
        mbar.add(procMenu);
        setMenuBar(mbar);

        Menu helpMenu = new Menu("Help", true);
        helpMenu.add(aboutItem = new MenuItem("About"));
        helpMenu.add(doTestFFT = new MenuItem("1D FFT Test"));
        mbar.add(helpMenu);
        setMenuBar(mbar);

    }

    public boolean handleEvent(Event event) {
        String filename;

        if (event.target instanceof MenuItem) {
            MenuItem item = (MenuItem) event.target;
            System.out.println(item.getLabel());

            //----------------------------------------------------------
            // Open a file
            //----------------------------------------------------------
            if (item == openItem) {
                menuSelectedOpen();
                return (true);
            }

            //----------------------------------------------------------
            // Save in PPM image format.
            //----------------------------------------------------------
            if (item == saveAsPPM) {
                menuSelectedSavePPM();
                return (true);
            }

            //----------------------------------------------------------
            // Save in GIF image format.
            //----------------------------------------------------------
            if (item == saveAsGIF) {
                menuSelectedSaveGIF();
                return (true);
            }




            //----------------------------------------------------------
            // Handle Translate
            //----------------------------------------------------------
            if (item == doTranslate) {
                new DoubleDialog("Translation dialog", "Enter factor:", ang);

                return (true);
            }

            //----------------------------------------------------------
            // Handle Histogram.
            //----------------------------------------------------------
            if (item == doHistogram) {
                // Make sure an image exists.
                if ((imageHeight != 0) && (imageWidth != 0)) {
                    new Histogram(imageData_R, "Red Component",
                            imageWidth, imageHeight, Color.red);
                    new Histogram(imageData_G, "Green Component",
                            imageWidth, imageHeight, Color.green);
                    new Histogram(imageData_B, "Blue Component",
                            imageWidth, imageHeight, Color.blue);

                    new Histogram(imageData_R, imageData_G,
                            imageData_B, "Intensity", imageWidth, imageHeight, Color.black);

                } else {
                    System.out.println("No histogram computed - an image needs to be loaded!");
                }

                return (true);
            }

            //----------------------------------------------------------
            // Handle FFT of Image.
            //----------------------------------------------------------
            if (item == doFFT) {
                menuSelectedFFT();
                return (true);
            }

            //----------------------------------------------------------
            // Handle IFFT of Image.
            //----------------------------------------------------------
            if (item == doIFFT) {
                menuSelectedIFFT();
                return (true);
            }


            //----------------------------------------------------------
            // Take 10 * Log PSD before displaying it.
            //----------------------------------------------------------
            if (item == doLogPSD) {
                imgFFT.DisplayLogPSD = true;
                return (true);
            }

            //----------------------------------------------------------
            // Display just PSD when displaying FFT data.
            //----------------------------------------------------------
            if (item == doNoLogPSD) {
                imgFFT.DisplayLogPSD = false;
                return (true);
            }

            //----------------------------------------------------------
            // Run a filter using FFT data from an image.
            //----------------------------------------------------------
            if (item == doExecuteFilter) {
                menuSelectedFilter();
                return (true);
            }

            //----------------------------------------------------------
            // Enhance Edges.
            //----------------------------------------------------------
            if (item == doEnhanceEdges) {
                menuEnhanceEdges();
                return (true);
            }

            //----------------------------------------------------------
            // Exit from application.
            //----------------------------------------------------------
            if (item == quitItem) {
                dispose();
                return (true);
            }

            //----------------------------------------------------------
            // Handle About menu selection.
            //----------------------------------------------------------
            if (item == aboutItem) {
                // TODO: about frame.
                return (true);
            }

            //----------------------------------------------------------
            // Handle 1-D FFT test to console.
            //----------------------------------------------------------
            if (item == doTestFFT) {
                menuSelectedTestFFT();
                return (true);
            }
        }
        return super.handleEvent(event);
    }

    public boolean keyDown(Event e, int key) {
        Image newImage;

        switch (key) {
            case 'r':
                double beta = ang.getValue();
                System.out.println("Angle: " + beta);

                imageData = iUtil.rotateImage(imageData, imageWidth, imageHeight, beta);
                imageData_R = CU.getRedArray(imageData);
                imageData_G = CU.getGreenArray(imageData);
                imageData_B = CU.getBlueArray(imageData);
                newImage = getToolkit().createImage(
                        new MemoryImageSource(imageWidth, imageHeight,
                                ColorModel.getRGBdefault(), imageData, 0, imageWidth));

                setLayout(new GridLayout(1, 1));
                if (imgScroll != null) {
                    // If there is already an image there delete it first
                    remove(imgScroll);
                }
                imgScroll = new ImageScroller(newImage);
                add(imgScroll);
                show();

                break;

            case 'z':
                int factor = (int) ang.getValue();
                System.out.println("Zoom percentage: " + factor);

                imageData = iUtil.zoomImage(imageData, imageWidth, imageHeight, factor);
                imageData_R = CU.getRedArray(imageData);
                imageData_G = CU.getGreenArray(imageData);
                imageData_B = CU.getBlueArray(imageData);
                newImage = getToolkit().createImage(
                        new MemoryImageSource(imageWidth, imageHeight,
                                ColorModel.getRGBdefault(), imageData, 0, imageWidth));

                setLayout(new GridLayout(1, 1));
                if (imgScroll != null) {
                    // If there is already an image there delete it first
                    remove(imgScroll);
                }
                imgScroll = new ImageScroller(newImage);
                add(imgScroll);
                show();

                break;
        }

        return (true);
    }

    private void menuSelectedOpen() {
        img = getImage();
        grabImageDisplayFrame();
    }

    private void grabPixels() {
        // getWidth and getHeight can use null observer
        // because we know that the image has been loaded.
        imageWidth = img.getWidth(this);
        imageHeight = img.getHeight(this);
        numPixels = imageWidth * imageHeight;
        System.out.println("Width: " + imageWidth);
        System.out.println("Height: " + imageHeight);
        System.out.println("Numpixels: " + numPixels);

        imageData = new int[numPixels];
        PixelGrabber grabber = new PixelGrabber(img,
                0, 0, imageWidth, imageHeight, imageData, 0, imageWidth);

        // Grab pixels
        try {
            if (grabber.grabPixels() != true) {
                System.err.println("Error while grabbing pixels");
                System.exit(1);
            }
        } catch (InterruptedException e) {
            System.out.println("PixelGrabber Exception!");
        }

        // separated R,G,B values
        imageData_R = CU.getRedArray(imageData);
        imageData_G = CU.getGreenArray(imageData);
        imageData_B = CU.getBlueArray(imageData);
    }

    private void grabImageDisplayFrame() {
        grabPixels();
        setLayout(new GridLayout(1, 1));
        if (imgScroll != null) {
            // If there is already an image scroller, delete it.
            remove(imgScroll);
        }
        imgScroll = new ImageScroller(img);
        add(imgScroll);
        show();
    }


    //----------------------------------------------------------------
    //
    // This menu selection opens an image file that will be used
    // as a mask on a 2D FFT array to perform edge detection or
    // other implement other image filters.
    //
    //----------------------------------------------------------------
    private void menuSelectedFilter() {
        int[] tempImageData;
        img = getImage();

        // The image we are reading now is just to use as a filter.
        // Anywhere that is black in the filter is zeroed in the FFT
        // data array.
        // getWidth and getHeight can use null observer
        // because we know that the image has been loaded.
        int imageWidth = img.getWidth(this);
        int imageHeight = img.getHeight(this);
        int numPixels = imageWidth * imageHeight;
        System.out.println("Filter width: " + imageWidth);
        System.out.println("Filter height: " + imageHeight);
        System.out.println("Filter numpixels: " + numPixels);

        tempImageData = new int[numPixels];
        PixelGrabber grabber = new PixelGrabber(img,
                0, 0, imageWidth, imageHeight, tempImageData, 0, imageWidth);

        // Grab pixels
        try {
            if (grabber.grabPixels() != true) {
                System.err.println("Error while grabbing pixels");
                System.exit(1);
            }
        } catch (InterruptedException e) {
            System.out.println("PixelGrabber Exception!");
        }

        // Each color pixel is a short.
        short[] imageData_R = CU.getRedArray(tempImageData);
        short[] imageData_G = CU.getGreenArray(tempImageData);
        short[] imageData_B = CU.getBlueArray(tempImageData);


        // Get the FFT array data that was calculated.
        float[][] cR_r = imgFFT.getRedReal();
        float[][] cR_i = imgFFT.getRedImaginary();
        float[][] cG_r = imgFFT.getGreenReal();
        float[][] cG_i = imgFFT.getGreenImaginary();
        float[][] cB_r = imgFFT.getBlueReal();
        float[][] cB_i = imgFFT.getBlueImaginary();

        for (int i = 0; i < imageHeight; i++) {
            for (int j = 0; j < imageWidth; j++) {
                // If the filter value is low zero the FFT data in
                // that region.
                if (imageData_R[i * imageWidth + j] < 10) {
                    cR_r[i][j] = 0;
                    cR_i[i][j] = 0;
                    cG_r[i][j] = 0;
                    cG_i[i][j] = 0;
                    cB_r[i][j] = 0;
                    cB_i[i][j] = 0;
                }
            }
        }

    }

    void menuEnhanceEdges() {
        for (int i = 0; i < numPixels; i++) {
            if (((imageData[i] & 0x000000ff) > 30) &&
                    ((imageData[i] & 0x0000ff00) > 30) &&
                    ((imageData[i] & 0x00ff0000) > 30)) {
                imageData[i] = 0xffffffff;
            } else {
                imageData[i] = 0xff000000;
            }
        }
        imageData_R = CU.getRedArray(imageData);
        imageData_G = CU.getGreenArray(imageData);
        imageData_B = CU.getBlueArray(imageData);

        // Create a Java Image
        Image newImage = getToolkit().createImage(
                new MemoryImageSource(imageWidth, imageHeight,
                        ColorModel.getRGBdefault(), imageData, 0, imageWidth));

        setLayout(new GridLayout(1, 1));
        if (imgScroll != null) {
            // If there is already an image there delete it first
            remove(imgScroll);
        }
        imgScroll = new ImageScroller(newImage);
        add(imgScroll);
        show();
    }

    //----------------------------------------------------------------
    //
    //
    //----------------------------------------------------------------
    private void menuSelectedSavePPM() {
        FileDialog fd = new FileDialog(this, "Save PPM Image", FileDialog.SAVE);
        fd.setFile("*.ppm");
        fd.show();
        String fname = fd.getFile();

        String fqname;

        if (fname != null) {
            if (fname.endsWith(".*.*")) {
                // Remove ".*.*" - bug in getfile()?
                fqname = fd.getDirectory() +
                        fname.substring(0, fname.length() - 4);
            } else {
                fqname = fd.getDirectory() + fname;
            }

            // Print image filename and directory to console
            System.out.println("Saving File: " + fqname);

            // See if this is a PPM save.
            if (fqname.endsWith(".ppm")) {

                // See how long it takes to write a file to disk.
                Timer t1 = new Timer();
                t1.mark();

                // Try PPM file writing.
                try {
                    WritePPM.DoIt(imageData_R, imageData_G, imageData_B,
                            imageWidth, imageHeight, fqname);
                } catch (Exception e) {
                    System.out.println("Save PPM Exception!");
                }

                // Stop the timer and report.
                t1.record();
                t1.report();
                fd.dispose();
            } else {
                System.out.println("Invalid filename: " + fqname);
            }
        }
    }

    //----------------------------------------------------------------
    //
    //
    //----------------------------------------------------------------
    private void menuSelectedSaveGIF() {
        FileDialog fd = new FileDialog(this, "Save GIF Image", FileDialog.SAVE);
        fd.setFile("*.gif");
        fd.show();
        String fname = fd.getFile();

        String fqname;

        if (fname != null) {
            if (fname.endsWith(".*.*")) {
                // Remove ".*.*" - bug in getfile()?
                fqname = fd.getDirectory() +
                        fname.substring(0, fname.length() - 4);
            } else {
                fqname = fd.getDirectory() + fname;
            }

            // Print image filename and directory to console
            System.out.println("Saving File: " + fqname);

            // See if this is a GIF save.
            if (fqname.endsWith(".gif")) {
                Image imageToWrite = getToolkit().createImage(
                        new MemoryImageSource(imageWidth, imageHeight,
                                ColorModel.getRGBdefault(), imageData, 0, imageWidth));

                // See how long it takes to write a file to disk.
                Timer t1 = new Timer();
                t1.mark();

                // Try GIF error.
                try {
                    WriteGIF.DoIt(imageToWrite, fqname);
                } catch (Exception e) {
                    System.out.println("Save GIF Exception!");
                }

                // Stop the timer and report.
                t1.record();
                t1.report();
                fd.dispose();
            } else {
                System.out.println("Invalid filename: " + fqname);
            }
        }
    }

    //----------------------------------------------------------------
    //
    //
    //----------------------------------------------------------------
    private void menuSelectedFFT() {
        // Make sure an image exists.
        if ((imageHeight == imageWidth) &&
                (imageHeight != 0)) {
            // Forward FFT on input image data in short arrays,
            // the complex data is returned in cX_r & cX_i arrays
            imageData = imgFFT.forward2dFFT(imageData_R, imageData_G,
                    imageData_B, imageWidth, imageHeight);

            imageData_R = CU.getRedArray(imageData);
            imageData_G = CU.getGreenArray(imageData);
            imageData_B = CU.getBlueArray(imageData);
            Image newImage = getToolkit().createImage(
                    new MemoryImageSource(imageWidth, imageHeight,
                            ColorModel.getRGBdefault(), imageData, 0, imageWidth));

            setLayout(new GridLayout(1, 1));
            if (imgScroll != null) {
                // If there is already an image there delete it first
                remove(imgScroll);
            }
            imgScroll = new ImageScroller(newImage);
            add(imgScroll);
            show();
        } else {
            System.out.println("No image loaded, or image not size not supported!");
        }
    }

    //----------------------------------------------------------------
    //
    //
    //----------------------------------------------------------------
    private void menuSelectedIFFT() {
        // Get the FFT array data that was calculated.
        float[][] cR_r = imgFFT.getRedReal();
        float[][] cR_i = imgFFT.getRedImaginary();
        float[][] cG_r = imgFFT.getGreenReal();
        float[][] cG_i = imgFFT.getGreenImaginary();
        float[][] cB_r = imgFFT.getBlueReal();
        float[][] cB_i = imgFFT.getBlueImaginary();

        // Make sure a complex representation of an image
        // has at least been allocated.
        if ((cR_r != null) && (cR_i != null) &&
                (cG_r != null) && (cG_i != null) &&
                (cB_r != null) && (cB_i != null)) {
            // Do the reverse FFT
            imageData = imgFFT.reverse2dFFT();
            imageData_R = CU.getRedArray(imageData);
            imageData_G = CU.getGreenArray(imageData);
            imageData_B = CU.getBlueArray(imageData);

            // Create a Java Image
            Image newImage = getToolkit().createImage(
                    new MemoryImageSource(imageWidth, imageHeight,
                            ColorModel.getRGBdefault(), imageData, 0, imageWidth));

            setLayout(new GridLayout(1, 1));
            if (imgScroll != null) {
                // If there is already an image there delete it first
                remove(imgScroll);
            }
            imgScroll = new ImageScroller(newImage);
            add(imgScroll);
            show();
        } else {
            System.out.println("No image loaded, or image not size not supported!");
        }
    }

    //----------------------------------------------------------------
    //
    //
    //----------------------------------------------------------------
    private void menuSelectedTestFFT() {
        imgFFT.TestFFT();

        FFT1D oneD = new FFT1D();
        oneD.TestFFT();
    }


}
